﻿/******************* 
 * Wm Task V1 Test *
 *******************/

import { PsychoJS } from './lib/core-2020.2.js';
import * as core from './lib/core-2020.2.js';
import { TrialHandler } from './lib/data-2020.2.js';
import { Scheduler } from './lib/util-2020.2.js';
import * as visual from './lib/visual-2020.2.js';
import * as sound from './lib/sound-2020.2.js';
import * as util from './lib/util-2020.2.js';
//some handy aliases as in the psychopy scripts;
const { abs, sin, cos, PI: pi, sqrt } = Math;

// init psychoJS:
const psychoJS = new PsychoJS({
  debug: true
});

// open window:
psychoJS.openWindow({
  fullscr: true,
  color: new util.Color('white'),
  units: 'height',
  waitBlanking: true
});

// store info about the experiment session:
let expName = 'WM Task v1';  // from the Builder filename that created this script
let expInfo = {'participant': '', 'session': '001'};

// schedule the experiment:
psychoJS.schedule(psychoJS.gui.DlgFromDict({
  dictionary: expInfo,
  title: expName
}));

const flowScheduler = new Scheduler(psychoJS);
const dialogCancelScheduler = new Scheduler(psychoJS);
psychoJS.scheduleCondition(function() { return (psychoJS.gui.dialogComponent.button === 'OK'); }, flowScheduler, dialogCancelScheduler);

// flowScheduler gets run if the participants presses OK
flowScheduler.add(updateInfo); // add timeStamp
flowScheduler.add(experimentInit);
flowScheduler.add(WM_InstructionsRoutineBegin());
flowScheduler.add(WM_InstructionsRoutineEachFrame());
flowScheduler.add(WM_InstructionsRoutineEnd());
const wm_trialsLoopScheduler = new Scheduler(psychoJS);
flowScheduler.add(wm_trialsLoopBegin, wm_trialsLoopScheduler);
flowScheduler.add(wm_trialsLoopScheduler);
flowScheduler.add(wm_trialsLoopEnd);
flowScheduler.add(EndRoutineBegin());
flowScheduler.add(EndRoutineEachFrame());
flowScheduler.add(EndRoutineEnd());
flowScheduler.add(quitPsychoJS, '', true);

// quit if user presses Cancel in dialog box:
dialogCancelScheduler.add(quitPsychoJS, '', false);

psychoJS.start({
  expName: expName,
  expInfo: expInfo,
  resources: [
  ]
});

function updateInfo() {
  expInfo['date'] = util.MonotonicClock.getDateStr();  // add a simple timestamp
  expInfo['expName'] = expName;
  expInfo['psychopyVersion'] = '2020.2.0';
  expInfo['OS'] = window.navigator.platform;

  // store frame rate of monitor if we can measure it successfully
  expInfo['frameRate'] = psychoJS.window.getActualFrameRate();
  if (typeof expInfo['frameRate'] !== 'undefined')
    frameDur = 1.0 / Math.round(expInfo['frameRate']);
  else
    frameDur = 1.0 / 60.0; // couldn't get a reliable measure so guess

  // add info from the URL:
  util.addInfoFromUrl(expInfo);
  
  return Scheduler.Event.NEXT;
}

function experimentInit() {
  // Initialize components for Routine "WM_Instructions"
  WM_InstructionsClock = new util.Clock();
  text = new visual.TextStim({
    win: psychoJS.window,
    name: 'text',
    text: 'In this experiment you will be presented with a single letter and then a video.\n\nAfter responding to the video, you will then be presented with a single letter and you will have to decide whether this letter is the same as or different from the letter presented at the beginning of the trial.\n\nRespond with the keys:\n - "Z" if the letter was the SAME \n - "/" if the letter was DIFFERENT \n\nYou will perform a total of 16 trials.  Try to respond as quickly and as accurately as possible.\n\nWhen you are ready to proceed press the spacebar.',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.04,  wrapWidth: undefined, ori: 0,
    color: new util.Color('black'),  opacity: 1,
    depth: 0.0 
  });
  
  ok = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  import * as random from 'random';
  var Videos, corrAns, firststim, pairlist, secondstim;
  Videos = ["Ethics_1sec_0-5_ped.mp4", "Ethics_1sec_1-5_ped.mp4", "Ethics_1sec_5-0_ped.mp4", "Ethics_1sec_5-1_ped.mp4", "Ethics_2sec_0-5_ped.mp4", "Ethics_2sec_1-5_ped.mp4", "Ethics_2sec_5-0_ped.mp4", "Ethics_2sec_5-1_ped.mp4", "Ethics_3sec_0-5_ped.mp4", "Ethics_3sec_1-5_ped.mp4", "Ethics_3sec_5-0_ped.mp4", "Ethics_3sec_5-1_ped.mp4", "NoPed_Catch1.mp4", "NoPed_Catch2.mp4", "NoPed_Catch3.mp4", "NoPed_Catch4.mp4"];
  random.shuffle(Videos);
  pairlist = [["D", "D", "z"], ["T", "T", "z"], ["G", "G", "z"], ["I", "I", "z"], ["E", "E", "z"], ["J", "J", "z"], ["F", "F", "z"], ["P", "P", "z"], ["Y", "K", "slash"], ["B", "T", "slash"], ["U", "N", "slash"], ["Z", "K", "slash"], ["K", "V", "slash"], ["N", "F", "slash"], ["V", "L", "slash"], ["O", "S", "slash"]];
  random.shuffle(pairlist);
  firststim = [];
  secondstim = [];
  corrAns = [];
  for (var i, _pj_c = 0, _pj_a = pairlist, _pj_b = _pj_a.length; (_pj_c < _pj_b); _pj_c += 1) {
      i = _pj_a[_pj_c];
      firststim.append(i[0]);
      secondstim.append(i[1]);
      corrAns.append(i[2]);
  }
  
  // Initialize components for Routine "WM_1"
  WM_1Clock = new util.Clock();
  presentSet = new visual.TextStim({
    win: psychoJS.window,
    name: 'presentSet',
    text: 'default text',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0,
    color: new util.Color('black'),  opacity: 1,
    depth: 0.0 
  });
  
  fixation1 = new visual.ShapeStim ({
    win: psychoJS.window, name: 'fixation1', 
    vertices: 'cross', size:[0.02, 0.02],
    ori: 0, pos: [0, 0],
    lineWidth: 0.5, lineColor: new util.Color('black'),
    fillColor: new util.Color('black'),
    opacity: 1, depth: -1, interpolate: true,
  });
  
  // Initialize components for Routine "trial"
  trialClock = new util.Clock();
  key_resp = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "WM_2"
  WM_2Clock = new util.Clock();
  fixation2 = new visual.ShapeStim ({
    win: psychoJS.window, name: 'fixation2', 
    vertices: 'cross', size:[0.02, 0.02],
    ori: 0, pos: [0, 0],
    lineWidth: 0.5, lineColor: new util.Color('black'),
    fillColor: new util.Color('black'),
    opacity: 1, depth: 0, interpolate: true,
  });
  
  presentProbe = new visual.TextStim({
    win: psychoJS.window,
    name: 'presentProbe',
    text: 'default text',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0,
    color: new util.Color('black'),  opacity: 1,
    depth: -1.0 
  });
  
  key_resp_2 = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "End"
  EndClock = new util.Clock();
  text_2 = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_2',
    text: 'Experiment Over\n\nThank you for your participation!',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.04,  wrapWidth: undefined, ori: 0,
    color: new util.Color('black'),  opacity: 1,
    depth: 0.0 
  });
  
  // Create some handy timers
  globalClock = new util.Clock();  // to track the time since experiment started
  routineTimer = new util.CountdownTimer();  // to track time remaining of each (non-slip) routine
  
  return Scheduler.Event.NEXT;
}

function WM_InstructionsRoutineBegin(snapshot) {
  return function () {
    //------Prepare to start Routine 'WM_Instructions'-------
    t = 0;
    WM_InstructionsClock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    ok.keys = undefined;
    ok.rt = undefined;
    _ok_allKeys = [];
    // keep track of which components have finished
    WM_InstructionsComponents = [];
    WM_InstructionsComponents.push(text);
    WM_InstructionsComponents.push(ok);
    
    for (const thisComponent of WM_InstructionsComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}

function WM_InstructionsRoutineEachFrame(snapshot) {
  return function () {
    //------Loop for each frame of Routine 'WM_Instructions'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = WM_InstructionsClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *text* updates
    if (t >= 0.0 && text.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text.tStart = t;  // (not accounting for frame time here)
      text.frameNStart = frameN;  // exact frame index
      
      text.setAutoDraw(true);
    }

    
    // *ok* updates
    if (t >= 0.0 && ok.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ok.tStart = t;  // (not accounting for frame time here)
      ok.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { ok.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { ok.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { ok.clearEvents(); });
    }

    if (ok.status === PsychoJS.Status.STARTED) {
      let theseKeys = ok.getKeys({keyList: ['space'], waitRelease: false});
      _ok_allKeys = _ok_allKeys.concat(theseKeys);
      if (_ok_allKeys.length > 0) {
        ok.keys = _ok_allKeys[_ok_allKeys.length - 1].name;  // just the last key pressed
        ok.rt = _ok_allKeys[_ok_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of WM_InstructionsComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}

function WM_InstructionsRoutineEnd(snapshot) {
  return function () {
    //------Ending Routine 'WM_Instructions'-------
    for (const thisComponent of WM_InstructionsComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    psychoJS.experiment.addData('ok.keys', ok.keys);
    if (typeof ok.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('ok.rt', ok.rt);
        routineTimer.reset();
        }
    
    ok.stop();
    // the Routine "WM_Instructions" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}

function wm_trialsLoopBegin(wm_trialsLoopScheduler) {
  // set up handler to look after randomisation of conditions etc
  wm_trials = new TrialHandler({
    psychoJS: psychoJS,
    nReps: 16, method: TrialHandler.Method.RANDOM,
    extraInfo: expInfo, originPath: undefined,
    trialList: undefined,
    seed: undefined, name: 'wm_trials'
  });
  psychoJS.experiment.addLoop(wm_trials); // add the loop to the experiment
  currentLoop = wm_trials;  // we're now the current loop

  // Schedule all the trials in the trialList:
  for (const thisWm_trial of wm_trials) {
    const snapshot = wm_trials.getSnapshot();
    wm_trialsLoopScheduler.add(importConditions(snapshot));
    wm_trialsLoopScheduler.add(WM_1RoutineBegin(snapshot));
    wm_trialsLoopScheduler.add(WM_1RoutineEachFrame(snapshot));
    wm_trialsLoopScheduler.add(WM_1RoutineEnd(snapshot));
    wm_trialsLoopScheduler.add(trialRoutineBegin(snapshot));
    wm_trialsLoopScheduler.add(trialRoutineEachFrame(snapshot));
    wm_trialsLoopScheduler.add(trialRoutineEnd(snapshot));
    wm_trialsLoopScheduler.add(WM_2RoutineBegin(snapshot));
    wm_trialsLoopScheduler.add(WM_2RoutineEachFrame(snapshot));
    wm_trialsLoopScheduler.add(WM_2RoutineEnd(snapshot));
    wm_trialsLoopScheduler.add(endLoopIteration(wm_trialsLoopScheduler, snapshot));
  }

  return Scheduler.Event.NEXT;
}

function wm_trialsLoopEnd() {
  psychoJS.experiment.removeLoop(wm_trials);

  return Scheduler.Event.NEXT;
}

function WM_1RoutineBegin(snapshot) {
  return function () {
    //------Prepare to start Routine 'WM_1'-------
    t = 0;
    WM_1Clock.reset(); // clock
    frameN = -1;
    routineTimer.add(3.500000);
    // update component parameters for each repeat
    presentSet.setText(firststim[wm_trials.thisN]);
    // keep track of which components have finished
    WM_1Components = [];
    WM_1Components.push(presentSet);
    WM_1Components.push(fixation1);
    
    for (const thisComponent of WM_1Components)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}

function WM_1RoutineEachFrame(snapshot) {
  return function () {
    //------Loop for each frame of Routine 'WM_1'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = WM_1Clock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *presentSet* updates
    if (t >= 2 && presentSet.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      presentSet.tStart = t;  // (not accounting for frame time here)
      presentSet.frameNStart = frameN;  // exact frame index
      
      presentSet.setAutoDraw(true);
    }

    frameRemains = 2 + 1.0 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (presentSet.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      presentSet.setAutoDraw(false);
    }
    
    // *fixation1* updates
    if (t >= 3.0 && fixation1.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      fixation1.tStart = t;  // (not accounting for frame time here)
      fixation1.frameNStart = frameN;  // exact frame index
      
      fixation1.setAutoDraw(true);
    }

    frameRemains = 3.0 + 0.5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (fixation1.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      fixation1.setAutoDraw(false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of WM_1Components)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}

function WM_1RoutineEnd(snapshot) {
  return function () {
    //------Ending Routine 'WM_1'-------
    for (const thisComponent of WM_1Components) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    return Scheduler.Event.NEXT;
  };
}

function trialRoutineBegin(snapshot) {
  return function () {
    //------Prepare to start Routine 'trial'-------
    t = 0;
    trialClock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    videosClock = new util.Clock();
    videos = new visual.MovieStim({
      win: psychoJS.window,
      name: 'videos',
      units: undefined,
      movie: Videos[wm_trials.thisN],
      pos: [0, 0],
      size: undefined,
      ori: 0,
      opacity: 1,
      loop: false,
      noAudio: false,
      });
    key_resp.keys = undefined;
    key_resp.rt = undefined;
    _key_resp_allKeys = [];
    // keep track of which components have finished
    trialComponents = [];
    trialComponents.push(videos);
    trialComponents.push(key_resp);
    
    for (const thisComponent of trialComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}

function trialRoutineEachFrame(snapshot) {
  return function () {
    //------Loop for each frame of Routine 'trial'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = trialClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *videos* updates
    if (t >= 0.0 && videos.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      videos.tStart = t;  // (not accounting for frame time here)
      videos.frameNStart = frameN;  // exact frame index
      
      videos.setAutoDraw(true);
      videos.play();
    }

    frameRemains = 0.0 + undefined - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (videos.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      videos.setAutoDraw(false);
    }

    if (videos.status === PsychoJS.Status.FINISHED) {  // force-end the routine
        continueRoutine = false;
    }
    
    // *key_resp* updates
    if (t >= 0.0 && key_resp.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_resp.tStart = t;  // (not accounting for frame time here)
      key_resp.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_resp.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_resp.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_resp.clearEvents(); });
    }

    if (key_resp.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_resp.getKeys({keyList: ['space'], waitRelease: false});
      _key_resp_allKeys = _key_resp_allKeys.concat(theseKeys);
      if (_key_resp_allKeys.length > 0) {
        key_resp.keys = _key_resp_allKeys[_key_resp_allKeys.length - 1].name;  // just the last key pressed
        key_resp.rt = _key_resp_allKeys[_key_resp_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of trialComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}

function trialRoutineEnd(snapshot) {
  return function () {
    //------Ending Routine 'trial'-------
    for (const thisComponent of trialComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    videos.stop();
    psychoJS.experiment.addData('key_resp.keys', key_resp.keys);
    if (typeof key_resp.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('key_resp.rt', key_resp.rt);
        routineTimer.reset();
        }
    
    key_resp.stop();
    thisExp.addData("video file", Videos[wm_trials.thisN]);
    
    // the Routine "trial" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}

function WM_2RoutineBegin(snapshot) {
  return function () {
    //------Prepare to start Routine 'WM_2'-------
    t = 0;
    WM_2Clock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    presentProbe.setText(secondstim[wm_trials.thisN]);
    key_resp_2.keys = undefined;
    key_resp_2.rt = undefined;
    _key_resp_2_allKeys = [];
    // keep track of which components have finished
    WM_2Components = [];
    WM_2Components.push(fixation2);
    WM_2Components.push(presentProbe);
    WM_2Components.push(key_resp_2);
    
    for (const thisComponent of WM_2Components)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}

function WM_2RoutineEachFrame(snapshot) {
  return function () {
    //------Loop for each frame of Routine 'WM_2'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = WM_2Clock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *fixation2* updates
    if (t >= 0.0 && fixation2.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      fixation2.tStart = t;  // (not accounting for frame time here)
      fixation2.frameNStart = frameN;  // exact frame index
      
      fixation2.setAutoDraw(true);
    }

    frameRemains = 0.0 + 0.5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (fixation2.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      fixation2.setAutoDraw(false);
    }
    
    // *presentProbe* updates
    if (t >= 0.5 && presentProbe.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      presentProbe.tStart = t;  // (not accounting for frame time here)
      presentProbe.frameNStart = frameN;  // exact frame index
      
      presentProbe.setAutoDraw(true);
    }

    
    // *key_resp_2* updates
    if (t >= 0.5 && key_resp_2.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_resp_2.tStart = t;  // (not accounting for frame time here)
      key_resp_2.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_resp_2.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_resp_2.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_resp_2.clearEvents(); });
    }

    if (key_resp_2.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_resp_2.getKeys({keyList: ['z', 'slash'], waitRelease: false});
      _key_resp_2_allKeys = _key_resp_2_allKeys.concat(theseKeys);
      if (_key_resp_2_allKeys.length > 0) {
        key_resp_2.keys = _key_resp_2_allKeys[_key_resp_2_allKeys.length - 1].name;  // just the last key pressed
        key_resp_2.rt = _key_resp_2_allKeys[_key_resp_2_allKeys.length - 1].rt;
        // was this correct?
        if (key_resp_2.keys == corrAns[wm_trials.thisN]) {
            key_resp_2.corr = 1;
        } else {
            key_resp_2.corr = 0;
        }
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of WM_2Components)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}

function WM_2RoutineEnd(snapshot) {
  return function () {
    //------Ending Routine 'WM_2'-------
    for (const thisComponent of WM_2Components) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // was no response the correct answer?!
    if (key_resp_2.keys === undefined) {
      if (['None','none',undefined].includes(corrAns[wm_trials.thisN])) {
         key_resp_2.corr = 1;  // correct non-response
      } else {
         key_resp_2.corr = 0;  // failed to respond (incorrectly)
      }
    }
    // store data for thisExp (ExperimentHandler)
    psychoJS.experiment.addData('key_resp_2.keys', key_resp_2.keys);
    psychoJS.experiment.addData('key_resp_2.corr', key_resp_2.corr);
    if (typeof key_resp_2.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('key_resp_2.rt', key_resp_2.rt);
        routineTimer.reset();
        }
    
    key_resp_2.stop();
    thisExp.addData("first stim", firststim[wm_trials.thisN]);
    thisExp.addData("second stim", secondstim[wm_trials.thisN]);
    
    // the Routine "WM_2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}

function EndRoutineBegin(snapshot) {
  return function () {
    //------Prepare to start Routine 'End'-------
    t = 0;
    EndClock.reset(); // clock
    frameN = -1;
    routineTimer.add(3.000000);
    // update component parameters for each repeat
    // keep track of which components have finished
    EndComponents = [];
    EndComponents.push(text_2);
    
    for (const thisComponent of EndComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}

function EndRoutineEachFrame(snapshot) {
  return function () {
    //------Loop for each frame of Routine 'End'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = EndClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *text_2* updates
    if (t >= 0.0 && text_2.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_2.tStart = t;  // (not accounting for frame time here)
      text_2.frameNStart = frameN;  // exact frame index
      
      text_2.setAutoDraw(true);
    }

    frameRemains = 0.0 + 3 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (text_2.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      text_2.setAutoDraw(false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of EndComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}

function EndRoutineEnd(snapshot) {
  return function () {
    //------Ending Routine 'End'-------
    for (const thisComponent of EndComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    return Scheduler.Event.NEXT;
  };
}

function endLoopIteration(scheduler, snapshot) {
  // ------Prepare for next entry------
  return function () {
    if (typeof snapshot !== 'undefined') {
      // ------Check if user ended loop early------
      if (snapshot.finished) {
        // Check for and save orphaned data
        if (psychoJS.experiment.isEntryEmpty()) {
          psychoJS.experiment.nextEntry(snapshot);
        }
        scheduler.stop();
      } else {
        const thisTrial = snapshot.getCurrentTrial();
        if (typeof thisTrial === 'undefined' || !('isTrials' in thisTrial) || thisTrial.isTrials) {
          psychoJS.experiment.nextEntry(snapshot);
        }
      }
    return Scheduler.Event.NEXT;
    }
  };
}

function importConditions(currentLoop) {
  return function () {
    psychoJS.importAttributes(currentLoop.getCurrentTrial());
    return Scheduler.Event.NEXT;
    };
}

function quitPsychoJS(message, isCompleted) {
  // Check for and save orphaned data
  if (psychoJS.experiment.isEntryEmpty()) {
    psychoJS.experiment.nextEntry();
  }
  
  
  
  
  
  
  psychoJS.window.close();
  psychoJS.quit({message: message, isCompleted: isCompleted});
  
  return Scheduler.Event.QUIT;
}
